@@ -19,7 +19,6 @@
  */
 
 
-
 int print_pan = 1;
 
 #include "fluidsynth_priv.h"
@@ -311,7 +310,9 @@ fluid_voice_write(fluid_voice_t* voice,
   fluid_real_t incr;
   fluid_real_t fres; 
 
-  /* All variables starting with dsp_ are used by the DSP loop. Documented in fluid_dsp_core.c */
+  /* All variables starting with dsp_ are used by the DSP
+     loop. Documented in fluid_dsp_core.c */
+
   int dsp_phase_index;
   unsigned int dsp_i;
   fluid_phase_t dsp_phase, dsp_phase_incr;
@@ -328,16 +329,20 @@ fluid_voice_write(fluid_voice_t* voice,
   int dsp_use_filter_flag = 1;
   short* dsp_data;
   int dsp_interp_method = voice->interp_method;
+
+
 #ifdef ENABLE_SSE
-  float mem_for_sse_interface[5*4+4];                              /* Reserve memory */
-                                                                   /* +4: add four floats for 16 added bytes */
-  sse_t* sse_n=(sse_t*)FLUID_ALIGN16BYTE(&mem_for_sse_interface);   /* Align the first element */
-  sse_t* sse_a=sse_n++;                                            /* The ++ operator increases
-								    * by the size of the structure! */
-  sse_t* sse_b=sse_n++;                                            
-  sse_t* sse_c=sse_n++;
-  sse_t* sse_d=sse_n++;
-  sse_t* sse_e=sse_n++;
+  float mem_for_sse_interface[5*4+4];             /* Reserve memory */
+                                                  /* +4: add four floats for 16 added bytes */
+
+  /* Align the first element */
+  sse_t* sse_n = (sse_t*) FLUID_ALIGN16BYTE(&mem_for_sse_interface);   
+  sse_t* sse_a = sse_n++;                         /* The ++ operator increases
+						   * by the size of the structure! */
+  sse_t* sse_b = sse_n++;
+  sse_t* sse_c = sse_n++;
+  sse_t* sse_d = sse_n++;
+  sse_t* sse_e = sse_n++;
   sse_t* sse_coeff;
 
   sse_t* sse_src;
@@ -345,13 +350,14 @@ fluid_voice_write(fluid_voice_t* voice,
   sse_t* sse_dest_right;
   sse_t* sse_dest;
 #endif
-  fluid_real_t dsp_buf_unaligned[FLUID_BUFSIZE+4];                    /* +4: add four floats for 16 added bytes */
-  fluid_real_t* dsp_buf=(fluid_real_t*)FLUID_ALIGN16BYTE(&dsp_buf_unaligned);
-/*  fluid_real_t dsp_sample; */
-  
+
+  /* +4: add four floats for 16 added bytes */
+  fluid_real_t dsp_buf_unaligned[FLUID_BUFSIZE+4];  
+  fluid_real_t* dsp_buf = (fluid_real_t*) FLUID_ALIGN16BYTE(&dsp_buf_unaligned);
   fluid_env_data_t* env_data;
   fluid_real_t x;
 
+
   /* make sure we're playing and that we have sample data */
   if (!_PLAYING(voice)) {
     return FLUID_OK;
@@ -359,9 +365,7 @@ fluid_voice_write(fluid_voice_t* voice,
 
   /******************* sample **********************/
 
-  if ((voice->sample == NULL)
-// || !voice->sample->valid
-) {
+  if (voice->sample == NULL) {
     fluid_voice_off(voice);
     return FLUID_OK;
   }
@@ -388,7 +392,7 @@ fluid_voice_write(fluid_voice_t* voice,
     x = env_data->min; 
     voice->volenv_section++;
     voice->volenv_count = 0;
-  } else if (x > env_data->max && 0) {
+  } else if (x > env_data->max) {
     x = env_data->max; 
     voice->volenv_section++;
     voice->volenv_count = 0;
@@ -399,7 +403,6 @@ fluid_voice_write(fluid_voice_t* voice,
 
   if (voice->volenv_section == FLUID_VOICE_ENVFINISHED) {
     fluid_profile(FLUID_PROF_VOICE_RELEASE, voice->ref);
-
     fluid_voice_off(voice);
     return FLUID_OK;
   }
@@ -469,7 +472,7 @@ fluid_voice_write(fluid_voice_t* voice,
 
   if (voice->volenv_section == FLUID_VOICE_ENVDELAY) {
     /* The volume amplitude is in hold phase. No sound is produced. */
-    goto amp_env_in_delay_phase_skip_DSP;
+    goto post_process;
   } else if (voice->volenv_section == FLUID_VOICE_ENVATTACK) {
     /* the envelope is in the attack section: ramp linearly to max value. */
     dsp_amp = (fluid_cb2amp(voice->attenuation + voice->modlfo_val * voice->modlfo_to_vol) 
@@ -495,28 +498,30 @@ fluid_voice_write(fluid_voice_t* voice,
     
     /* A voice can be turned off, when an estimate for the volume
      * (upper bound) falls below that volume, that will drop the
-     * sample below the noise floor.
-     * Start out with assuming a 0 dB sample... */
+     * sample below the noise floor.  Start out with assuming a 0 dB
+     * sample... */
     amplitude_that_reaches_noise_floor = FLUID_NOISE_FLOOR;
     if (voice->has_looped) {
-      /* For the loop we may have a better estimate for the volume
-       * of the actual sample (from peak detector): */
+      /* For the loop we may have a better estimate for the volume of
+       * the actual sample (from peak detector): */
       amplitude_that_reaches_noise_floor = voice->amplitude_that_reaches_noise_floor;
     }
     
     /* voice->attenuation_min is a lower boundary for the attenuation
-     * now and in the future (possibly 0 in the worst case).
-     * Now the amplitude of sample and volenv cannot
-     * exceed amp_max (since volenv_val can only drop):
+     * now and in the future (possibly 0 in the worst case).  Now the
+     * amplitude of sample and volenv cannot exceed amp_max (since
+     * volenv_val can only drop):
      */
     amp_max = fluid_cb2amp(voice->min_attenuation_cB
 			  + 960.0f * (1.0f - voice->volenv_val));
+
     /*    printf("Att min: %f Amp max: %f Limit: %f\n",voice->min_attenuation_cB,amp_max, amplitude_that_reaches_noise_floor); */
     
 //    printf("Amp max: %f\n",amp_max);
+
     /* And if amp_max is already smaller than the known amplitude,
-     * which will attenuate the sample below the noise floor, then
-     * we can safely turn off the voice. Duh. */
+     * which will attenuate the sample below the noise floor, then we
+     * can safely turn off the voice. Duh. */
     if (amp_max < amplitude_that_reaches_noise_floor){
       fluid_profile(FLUID_PROF_VOICE_RELEASE, voice->ref);
 /*            printf("Voice turned off! Amp is %f\n", amp_max); */
@@ -526,15 +531,16 @@ fluid_voice_write(fluid_voice_t* voice,
   }
   
   /* At this point, dsp_amp is the desired amplitude for the voice.
-   * This value will be reached at the end of the next buffer.
-   * So we raise the amplitude smoothly from the current voice
-   * amplitude (voice->amp) to the wanted amplitude dsp_amp.
+   * This value will be reached at the end of the next buffer.  So we
+   * raise the amplitude smoothly from the current voice amplitude
+   * (voice->amp) to the wanted amplitude dsp_amp.
    *
-   * By how much do we have to increase voice->amp, so that it reaches dsp_amp, when the
-   * increment is added FLUID_BUFSIZE times? */
+   * By how much do we have to increase voice->amp, so that it reaches
+   * dsp_amp, when the increment is added FLUID_BUFSIZE times? */
   dsp_amp_incr = (dsp_amp - voice->amp) / FLUID_BUFSIZE;
-  /* dsp_amp will now be fed into the DSP loop. 
-   * Use the amplitude, that came out of the last DSP loop run. */
+
+  /* dsp_amp will now be fed into the DSP loop.  Use the amplitude,
+   * that came out of the last DSP loop run. */
   dsp_amp = voice->amp;
   
   fluid_check_fpe("voice_write amplitude calculation");
@@ -542,25 +548,28 @@ fluid_voice_write(fluid_voice_t* voice,
     goto post_process;    
   }
 
-  /* Calculate the number of samples, that the DSP loop advances through the original waveform 
-   * with each step in the output buffer. It is the ratio between the frequencies of original
+  /* Calculate the number of samples, that the DSP loop advances
+   * through the original waveform with each step in the output
+   * buffer. It is the ratio between the frequencies of original
    * waveform and output waveform.*/
   incr = fluid_ct2hz(voice->pitch
 		    + voice->modlfo_val * voice->modlfo_to_pitch
 		    + voice->viblfo_val * voice->viblfo_to_pitch
 		    + voice->modenv_val * voice->modenv_to_pitch) / voice->root_pitch;
 
-  /* Transfer the phase from the voice into the dsp loop parameter dsp_phase */
+  /* Transfer the phase from the voice into the dsp loop parameter
+     dsp_phase */
   fluid_phase_set(dsp_phase, voice->phase);
 
-  /* Convert the 'speed' through the original waveform to a representation 'integer part'
-   * and 'fractional part' */
+  /* Convert the 'speed' through the original waveform to a
+   * representation 'integer part' and 'fractional part' */
   fluid_phase_set_float(dsp_phase_incr, incr);
   incr *= 1.00000000000001; /* gcc optimization problem. Quick fix. */
   fluid_check_fpe("voice_write phase calculation");
 
-  /* Check, if we are really making progress through the original sample.
-   * If the step size is rounded to 0, the DSP loop would get stuck. */
+  /* Check, if we are really making progress through the original
+   * sample.  If the step size is rounded to 0, the DSP loop would get
+   * stuck. */
   if (fluid_phase_index(dsp_phase) == 0 && fluid_phase_fract(dsp_phase) == 0){
       fluid_phase_fract(dsp_phase)=1;
   }
@@ -571,6 +580,7 @@ fluid_voice_write(fluid_voice_t* voice,
   fres = fluid_ct2hz(voice->fres 
 		    + voice->modlfo_val * voice->modlfo_to_fc 
 		    + voice->modenv_val * voice->modenv_to_fc);
+
   /* Testcase for filter resonance: Use the following line, and press
    * A3. There should be a _very_ pronounced resonant peak, which
    * drops down, when moving the pitch bend wheel only slightly. */
@@ -578,34 +588,34 @@ fluid_voice_write(fluid_voice_t* voice,
 
   if (fres < 5) {
       fres = 5;
-  } else if (fres > FLUID_MAX_AUDIBLE_FILTER_FC
-	     && voice->q_lin < FLUID_MIN_AUDIBLE_FILTER_Q) {
-      /* Beyond a certain point the filter does not change the sound much.
-       * Turn it off to save CPU cycles. */
-      /* Fixme: When the filter has been on once for a voice, keep it on forever. */
-      dsp_use_filter_flag=0;
+  } else if ((fres > FLUID_MAX_AUDIBLE_FILTER_FC)
+	     && (voice->q_lin < FLUID_MIN_AUDIBLE_FILTER_Q)) {
+    /* Beyond a certain point the filter does not change the sound
+     * much.  Turn it off to save CPU cycles. */
+    /* Fixme: When the filter has been on once for a voice, keep it on forever. */
+    dsp_use_filter_flag = 0;
   }
    
-   if (dsp_use_filter_flag && (abs(fres-voice->last_fres)>0.01)) {
+   if (dsp_use_filter_flag && (abs(fres - voice->last_fres) > 0.01)) {
      
-     /* The filter coefficients have to be recalculated (filter parameters
-      * have changed). Recalculation for various reasons is forced by 
-      * setting last_fres to -1.
-      * The flag filter_startup indicates, that the DSP loop runs
-      * for the first time, in this case, the filter is set directly,
-      * instead of smoothly fading between old and new settings.
+     /* The filter coefficients have to be recalculated (filter
+      * parameters have changed). Recalculation for various reasons is
+      * forced by setting last_fres to -1.  The flag filter_startup
+      * indicates, that the DSP loop runs for the first time, in this
+      * case, the filter is set directly, instead of smoothly fading
+      * between old and new settings.
       *
       * Those equations from Robert Bristow-Johnson's `Cookbook
       * formulae for audio EQ biquad filter coefficients', obtained
-      * from Harmony-central.com / Computer / Programming They are
+      * from Harmony-central.com / Computer / Programming. They are
       * the result of the bilinear transform on an analogue filter
       * prototype. To quote, `BLT frequency warping has been taken
-      * into account * for both significant frequency relocation and
-      * for bandwidth readjustment'. */
+      * into account for both significant frequency relocation and for
+      * bandwidth readjustment'. */
      
      fluid_real_t omega = (fluid_real_t) (2.0 * M_PI * (fres / 44100.0f));
-     fluid_real_t sin_coeff = (fluid_real_t) (sin(omega));
-     fluid_real_t cos_coeff = (fluid_real_t) (cos(omega));
+     fluid_real_t sin_coeff = (fluid_real_t) sin(omega);
+     fluid_real_t cos_coeff = (fluid_real_t) cos(omega);
      fluid_real_t alpha_coeff = sin_coeff / (2.0f * voice->q_lin);
      fluid_real_t a0_inv = 1.0f / (1.0f + alpha_coeff); 
      
@@ -631,25 +641,29 @@ fluid_voice_write(fluid_voice_t* voice,
        voice->a1 = a1_temp;
        voice->a2 = a2_temp;
        voice->b02 = b02_temp;
-       voice->b1 = b1_temp;
-       voice->filter_coeff_incr_count=0;
-       voice->filter_startup=0;
+       voice->b1 = b1_temp; 
+       voice->filter_coeff_incr_count = 0;
+       voice->filter_startup = 0;
 //       printf("Setting initial filter coefficients.\n");
      } else {
-       /* The filter frequency is changed.
-	* Calculate an increment factor, so that the new setting is reached
-	* after one buffer length. x_incr is added to the current value FLUID_BUFSIZE
+
+       /* The filter frequency is changed.  Calculate an increment
+	* factor, so that the new setting is reached after one buffer
+	* length. x_incr is added to the current value FLUID_BUFSIZE
 	* times. The length is arbitrarily chosen. Longer than one
-	* buffer will sacrifice some performance, though.
-	* Note: If the filter is still too 'grainy', then increase this number at will.
+	* buffer will sacrifice some performance, though.  Note: If
+	* the filter is still too 'grainy', then increase this number
+	* at will.
 	*/
-#define FILTER_TRANSITION_SAMPLES (2*FLUID_BUFSIZE)
-       voice->a1_incr=(a1_temp-voice->a1)*(1./(fluid_real_t)(FILTER_TRANSITION_SAMPLES));
-       voice->a2_incr=(a2_temp-voice->a2)*(1./(fluid_real_t)(FILTER_TRANSITION_SAMPLES));
-       voice->b02_incr=(b02_temp-voice->b02)*(1./(fluid_real_t)(FILTER_TRANSITION_SAMPLES));
-       voice->b1_incr=(b1_temp-voice->b1)*(1./(fluid_real_t)(FILTER_TRANSITION_SAMPLES));
+
+#define FILTER_TRANSITION_SAMPLES (FLUID_BUFSIZE)
+
+       voice->a1_incr = (a1_temp - voice->a1) / FILTER_TRANSITION_SAMPLES;
+       voice->a2_incr = (a2_temp - voice->a2) / FILTER_TRANSITION_SAMPLES;
+       voice->b02_incr = (b02_temp - voice->b02) / FILTER_TRANSITION_SAMPLES;
+       voice->b1_incr = (b1_temp - voice->b1) / FILTER_TRANSITION_SAMPLES;
        /* Have to add the increments filter_coeff_incr_count times. */
-       voice->filter_coeff_incr_count=(FILTER_TRANSITION_SAMPLES);
+       voice->filter_coeff_incr_count = FILTER_TRANSITION_SAMPLES;
      }
      voice->last_fres = fres;
      fluid_check_fpe("voice_write filter calculation");
@@ -670,6 +684,10 @@ fluid_voice_write(fluid_voice_t* voice,
    dsp_a2 = voice->a2;
    dsp_b02 = voice->b02;
    dsp_b1 = voice->b1;
+   dsp_a1_incr = voice->a1_incr;
+   dsp_a2_incr = voice->a2_incr;
+   dsp_b02_incr = voice->b02_incr;
+   dsp_b1_incr = voice->b1_incr;
    dsp_filter_coeff_incr_count = voice->filter_coeff_incr_count;
    
    fluid_check_fpe("voice_write DSP coefficients");
@@ -681,11 +699,12 @@ fluid_voice_write(fluid_voice_t* voice,
    * may require several runs. */
   fluid_check_fpe("voice_write DSP processing");
 
-  if (
-    ((_SAMPLEMODE(voice) == FLUID_LOOP) && (voice->volenv_section < FLUID_VOICE_ENVRELEASE)) 
-    || (_SAMPLEMODE(voice) == FLUID_LOOP_DURING_RELEASE)) {
-    /* At which index does the loop point occur in the output buffer? 
-     * This calculates the first index in the buffer, which uses sample data taken after the looparound. */
+  if (((_SAMPLEMODE(voice) == FLUID_LOOP) && (voice->volenv_section < FLUID_VOICE_ENVRELEASE)) 
+      || (_SAMPLEMODE(voice) == FLUID_LOOP_DURING_RELEASE)) {
+
+    /* At which index does the loop point occur in the output buffer?
+     * This calculates the first index in the buffer, which uses
+     * sample data taken after the looparound. */
     end_in_buffer = fluid_phase_steps(dsp_phase, voice->loop_end_offset, incr);
     
     if (end_in_buffer >= FLUID_BUFSIZE) {
@@ -694,20 +713,21 @@ fluid_voice_write(fluid_voice_t* voice,
       dsp_start = 0;
       dsp_end = FLUID_BUFSIZE;
 #include "fluid_dsp_core.c"
+
     } else {
-      /* The loop occurs during the current buffer length. 
-      * Note, that this method is unable to cope with a 'skipped' loop point.
-      * If, by means of witchcraft, evil magic or modulators, we end up beyond
-      * the loop point, a SEGV is unavoidable.
-      * We can't even detect this here, because the calculations are done using
-      * unsigned ints (only positive), and end_in_buffer would be negative
-      * for a skipped loop point...*/ 
+      /* The loop occurs during the current buffer length. Note, that
+       * this method is unable to cope with a 'skipped' loop point.  *
+       * If, by means of witchcraft, evil magic or modulators, we end
+       * up beyond * the loop point, a SEGV is unavoidable.  * We
+       * can't even detect this here, because the calculations are
+       * done using * unsigned ints (only positive), and end_in_buffer
+       * would be negative * for a skipped loop point...*/ 
       start = 0;
 
       while (end_in_buffer < FLUID_BUFSIZE) {
 
-	dsp_start=start;
-	dsp_end=end_in_buffer;
+	dsp_start = start;
+	dsp_end = end_in_buffer;
 #include "fluid_dsp_core.c"
       
 	/* loop */
@@ -716,26 +736,26 @@ fluid_voice_write(fluid_voice_t* voice,
 	start = end_in_buffer;
 	end_in_buffer += fluid_phase_steps(dsp_phase, voice->loop_end_offset, incr);
       }
-      dsp_start=start;
-      dsp_end=FLUID_BUFSIZE;
+      dsp_start = start;
+      dsp_end = FLUID_BUFSIZE;
 #include "fluid_dsp_core.c"
     }
     
   } else {
     /* Not looping right now. */
     
-    dsp_start=0;
+    dsp_start = 0;
     end_in_buffer = fluid_phase_steps(dsp_phase, voice->end_offset, incr);
 
     if (end_in_buffer >= FLUID_BUFSIZE) {
       /* Run the whole buffer at once */
-      dsp_end=FLUID_BUFSIZE;
+      dsp_end = FLUID_BUFSIZE;
 #include "fluid_dsp_core.c"
     } else {
       /* The sample ends in the middle of the buffer length. 
        * Process that far, and turn the voice off. 
        */
-      dsp_end=end_in_buffer;
+      dsp_end = end_in_buffer;
 #include "fluid_dsp_core.c"
       fluid_profile(FLUID_PROF_VOICE_RELEASE, voice->ref);
       fluid_voice_off(voice);
@@ -750,8 +770,17 @@ fluid_voice_write(fluid_voice_t* voice,
   voice->hist2 = dsp_hist2;
   voice->phase = dsp_phase;
   voice->amp = dsp_amp;
+  voice->a1 = dsp_a1;
+  voice->a2 = dsp_a2;
+  voice->b02 = dsp_b02;
+  voice->b1 = dsp_b1;
+  voice->filter_coeff_incr_count = dsp_filter_coeff_incr_count;
+
+/*   if (dsp_filter_coeff_incr_count) { */
+/*     printf("ticks = %d, dsp_filter_coeff_incr_count = %d\n",  */
+/* 	   voice->ticks, dsp_filter_coeff_incr_count); */
+/*   } */
 
- amp_env_in_delay_phase_skip_DSP:
  post_process:
   voice->ticks += FLUID_BUFSIZE;
   fluid_check_fpe("voice_write postprocess");
@@ -1078,12 +1107,9 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
   case GEN_FILTERFC:
     /* The resonance frequency is converted from absolute cents to midicents
      * .val and .mod are both used, this permits real-time modulation.
+     * The allowed range is tested in the 'fluid_ct2hz' function [PH,20021214] 
      */
     voice->fres = _GEN(voice, GEN_FILTERFC);
-		
-    /* Range: SF2.01 section 8.1.3 # 8 (note that these numbers are NOT frequencies) */
-    /* The allowed range is tested in the 'fluid_ct2hz' function [PH,20021214] */
-/*      fluid_clip(voice->fres, 1500.0f, 13500.0f); */
 
     /* The synthesis loop will have to recalculate the filter coefficients. */
     voice->last_fres = -1.0f;
@@ -1113,19 +1139,20 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
      * follows: */
     q_dB -= 3.01f;
 	    
-    /* The 'sound font' Q is defined in dB. The filter needs a linear q. Convert. */
+    /* The 'sound font' Q is defined in dB. The filter needs a linear
+       q. Convert. */
     voice->q_lin = (fluid_real_t) (pow(10.0f, q_dB / 20.0f));
 
     /* SF 2.01 page 59:
-     *  The SoundFont specs ask for a gain reduction equal to half
-     *  the height of the resonance peak (Q).  For example, for a
-     *  10 dB resonance peak, the gain is reduced by 5 dB.  This is
-     *  done by multiplying the total gain with sqrt(1/Q).  `Sqrt'
-     *  divides dB by 2 (100 lin = 40 dB, 10 lin = 20 dB, 3.16 lin
-     *  = 10 dB etc) The gain is later factored into the 'b' 
-     *  coefficients (numerator of the filter equation).
-     *  This gain factor depends only on Q, so this is the right place to
-     *  calculate it.
+     *
+     *  The SoundFont specs ask for a gain reduction equal to half the
+     *  height of the resonance peak (Q).  For example, for a 10 dB
+     *  resonance peak, the gain is reduced by 5 dB.  This is done by
+     *  multiplying the total gain with sqrt(1/Q).  `Sqrt' divides dB
+     *  by 2 (100 lin = 40 dB, 10 lin = 20 dB, 3.16 lin = 10 dB etc)
+     *  The gain is later factored into the 'b' coefficients
+     *  (numerator of the filter equation).  This gain factor depends
+     *  only on Q, so this is the right place to calculate it.
      */
     voice->filter_gain = (fluid_real_t) (1.0 / sqrt(voice->q_lin));
 		
@@ -1341,7 +1368,7 @@ fluid_voice_update_param(fluid_voice_t* voice, int gen)
     voice->volenv_data[FLUID_VOICE_ENVRELEASE].coeff = 1.0f;
     voice->volenv_data[FLUID_VOICE_ENVRELEASE].incr = count ? -1.0f / count : 0.0f;
     voice->volenv_data[FLUID_VOICE_ENVRELEASE].min = 0.0f;
-    voice->volenv_data[FLUID_VOICE_ENVRELEASE].max = 2.0f;
+    voice->volenv_data[FLUID_VOICE_ENVRELEASE].max = 1.0f;
     break;
     
     /* Modulation envelope */
@@ -1882,9 +1909,9 @@ void fluid_voice_check_sample_sanity(fluid_voice_t* voice)
     
     /* Keep start and end point in the right order */
     if (voice->start_offset > voice->end_offset){
-	int temp=voice->start_offset;
-	voice->start_offset=voice->end_offset;
-	voice->end_offset=temp;
+	int temp = voice->start_offset;
+	voice->start_offset = voice->end_offset;
+	voice->end_offset = temp;
 	/*FLUID_LOG(FLUID_DBG, "Loop / sample sanity check: Changing order of start / end points!"); */
     }
     
